@@ -36,10 +36,22 @@
 #include <gdk-pixbuf/gdk-pixdata.h>
 
 #include "xfce-backdrop.h"
+#include "xfce-desktop-enum-types.h"
+#include "xfdesktop-common.h"  /* for DEFAULT_BACKDROP */
+
+#ifndef abs
+#define abs(x)  ( (x) < 0 ? -(x) : (x) )
+#endif
 
-static void xfce_backdrop_class_init(XfceBackdropClass *klass);
-static void xfce_backdrop_init(XfceBackdrop *backdrop);
 static void xfce_backdrop_finalize(GObject *object);
+static void xfce_backdrop_set_property(GObject *object,
+                                       guint property_id,
+                                       const GValue *value,
+                                       GParamSpec *pspec);
+static void xfce_backdrop_get_property(GObject *object,
+                                       guint property_id,
+                                       GValue *value,
+                                       GParamSpec *pspec);
 
 struct _XfceBackdropPriv
 {
@@ -55,14 +67,29 @@ struct _XfceBackdropPriv
     gchar *image_path;
     
     gint brightness;
+    gdouble saturation;
 };
 
-enum {
+enum
+{
     BACKDROP_CHANGED,
-    LAST_SIGNAL
+    LAST_SIGNAL,
 };
 
-static guint backdrop_signals[LAST_SIGNAL] = { 0 };
+enum
+{
+    PROP_0 = 0,
+    PROP_COLOR_STYLE,
+    PROP_COLOR1,
+    PROP_COLOR2,
+    PROP_SHOW_IMAGE,
+    PROP_IMAGE_STYLE,
+    PROP_IMAGE_FILENAME,
+    PROP_BRIGHTNESS,
+    PROP_SATURATION,
+};
+
+static guint backdrop_signals[LAST_SIGNAL] = { 0, };
 
 /* helper functions */
 
@@ -113,15 +140,19 @@ adjust_brightness(GdkPixbuf *src, gint amount)
 }
 
 static GdkPixbuf *
-create_solid(GdkColor *color, gint width, gint height)
+create_solid(GdkColor *color,
+             gint width,
+             gint height,
+             gboolean has_alpha,
+             gint alpha)
 {
     GdkPixbuf *pix;
     guint32 rgba;
     
-    pix = gdk_pixbuf_new(GDK_COLORSPACE_RGB, FALSE, 8, width, height);
+    pix = gdk_pixbuf_new(GDK_COLORSPACE_RGB, has_alpha, 8, width, height);
     
-    rgba = (((color->red & 0xff00) << 8) | ((color->green & 0xff00))
-            | ((color->blue & 0xff00) >> 8)) << 8;
+    rgba = ((((color->red & 0xff00) << 8) | ((color->green & 0xff00))
+            | ((color->blue & 0xff00) >> 8)) << 8) | alpha;
     
     gdk_pixbuf_fill(pix, rgba);
     
@@ -164,7 +195,9 @@ create_gradient(GdkColor *color1, GdkColor *color2, gint width, gint height,
             memcpy(pixdata.pixel_data+(i*pixdata.rowstride),
                     pixdata.pixel_data, pixdata.rowstride);
         }
-    } else {
+    } else if(XFCE_BACKDROP_COLOR_TRANSPARENT == style)
+        memset(pixdata.pixel_data, 0x0, width * height * 3);
+    else {
         for(i = 0; i < height; i++) {
             rgb[0] = (color1->red + (i * (color2->red - color1->red) / height)) >> 8;
             rgb[1] = (color1->green + (i * (color2->green - color1->green) / height)) >> 8;
@@ -200,11 +233,79 @@ xfce_backdrop_class_init(XfceBackdropClass *klass)
     g_type_class_add_private(klass, sizeof(XfceBackdropPriv));
     
     gobject_class->finalize = xfce_backdrop_finalize;
+    gobject_class->set_property = xfce_backdrop_set_property;
+    gobject_class->get_property = xfce_backdrop_get_property;
     
     backdrop_signals[BACKDROP_CHANGED] = g_signal_new("changed",
             G_OBJECT_CLASS_TYPE(gobject_class), G_SIGNAL_RUN_FIRST,
             G_STRUCT_OFFSET(XfceBackdropClass, changed), NULL, NULL,
             g_cclosure_marshal_VOID__VOID, G_TYPE_NONE, 0);
+
+#define XFDESKTOP_PARAM_FLAGS  (G_PARAM_READWRITE \
+                                | G_PARAM_CONSTRUCT \
+                                | G_PARAM_STATIC_NAME \
+                                | G_PARAM_STATIC_NICK \
+                                | G_PARAM_STATIC_BLURB)
+
+    g_object_class_install_property(gobject_class, PROP_COLOR_STYLE,
+                                    g_param_spec_enum("color-style",
+                                                      "color style",
+                                                      "color style",
+                                                      XFCE_TYPE_BACKDROP_COLOR_STYLE,
+                                                      XFCE_BACKDROP_COLOR_SOLID,
+                                                      XFDESKTOP_PARAM_FLAGS));
+
+    g_object_class_install_property(gobject_class, PROP_COLOR1,
+                                    g_param_spec_boxed("first-color",
+                                                       "first color",
+                                                       "first color",
+                                                       GDK_TYPE_COLOR,
+                                                       XFDESKTOP_PARAM_FLAGS));
+
+    g_object_class_install_property(gobject_class, PROP_COLOR2,
+                                    g_param_spec_boxed("second-color",
+                                                       "second color",
+                                                       "second color",
+                                                       GDK_TYPE_COLOR,
+                                                       XFDESKTOP_PARAM_FLAGS));
+
+    g_object_class_install_property(gobject_class, PROP_SHOW_IMAGE,
+                                    g_param_spec_boolean("show-image",
+                                                         "show image",
+                                                         "show image",
+                                                         TRUE,
+                                                         XFDESKTOP_PARAM_FLAGS));
+
+    g_object_class_install_property(gobject_class, PROP_IMAGE_STYLE,
+                                    g_param_spec_enum("image-style",
+                                                      "image style",
+                                                      "image style",
+                                                      XFCE_TYPE_BACKDROP_IMAGE_STYLE,
+                                                      XFCE_BACKDROP_IMAGE_AUTO,
+                                                      XFDESKTOP_PARAM_FLAGS));
+
+    g_object_class_install_property(gobject_class, PROP_IMAGE_FILENAME,
+                                    g_param_spec_string("image-filename",
+                                                        "image filename",
+                                                        "image filename",
+                                                        DEFAULT_BACKDROP,
+                                                        XFDESKTOP_PARAM_FLAGS));
+
+    g_object_class_install_property(gobject_class, PROP_BRIGHTNESS,
+                                    g_param_spec_int("brightness",
+                                                     "brightness",
+                                                     "brightness",
+                                                     -128, 127, 0,
+                                                     XFDESKTOP_PARAM_FLAGS));
+
+    g_object_class_install_property(gobject_class, PROP_SATURATION,
+                                    g_param_spec_double("saturation",
+                                                        "saturation",
+                                                        "saturation",
+                                                        -10.0, 10.0, 1.0,
+                                                        XFDESKTOP_PARAM_FLAGS));
+
+#undef XFDESKTOP_PARAM_FLAGS
 }
 
 static void
@@ -213,6 +314,14 @@ xfce_backdrop_init(XfceBackdrop *backdrop)
     backdrop->priv = G_TYPE_INSTANCE_GET_PRIVATE(backdrop, XFCE_TYPE_BACKDROP,
                                                  XfceBackdropPriv);
     backdrop->priv->show_image = TRUE;
+
+    /* color defaults */
+    backdrop->priv->color1.red = 0x1515;
+    backdrop->priv->color1.green = 0x2222;
+    backdrop->priv->color1.blue = 0x3333;
+    backdrop->priv->color2.red = 0x1515;
+    backdrop->priv->color2.green = 0x2222;
+    backdrop->priv->color2.blue = 0x3333;
 }
 
 static void
@@ -228,6 +337,108 @@ xfce_backdrop_finalize(GObject *object)
     G_OBJECT_CLASS(xfce_backdrop_parent_class)->finalize(object);
 }
 
+static void
+xfce_backdrop_set_property(GObject *object,
+                           guint property_id,
+                           const GValue *value,
+                           GParamSpec *pspec)
+{
+    XfceBackdrop *backdrop = XFCE_BACKDROP(object);
+    GdkColor *color;
+
+    switch(property_id) {
+        case PROP_COLOR_STYLE:
+            xfce_backdrop_set_color_style(backdrop, g_value_get_enum(value));
+            break;
+
+        case PROP_COLOR1:
+            color = g_value_get_boxed(value);
+            if(color)
+                xfce_backdrop_set_first_color(backdrop, color);
+            break;
+
+        case PROP_COLOR2:
+            color = g_value_get_boxed(value);
+            if(color)
+                xfce_backdrop_set_second_color(backdrop, color);
+            break;
+
+        case PROP_SHOW_IMAGE:
+            xfce_backdrop_set_show_image(backdrop, g_value_get_boolean(value));
+            break;
+
+        case PROP_IMAGE_STYLE:
+            xfce_backdrop_set_image_style(backdrop, g_value_get_enum(value));
+            break;
+
+        case PROP_IMAGE_FILENAME:
+            xfce_backdrop_set_image_filename(backdrop,
+                                             g_value_get_string(value));
+            break;
+
+        case PROP_BRIGHTNESS:
+            xfce_backdrop_set_brightness(backdrop, g_value_get_int(value));
+            break;
+
+        case PROP_SATURATION:
+            xfce_backdrop_set_saturation(backdrop, g_value_get_double(value));
+            break;
+
+        default:
+            G_OBJECT_WARN_INVALID_PROPERTY_ID(object, property_id, pspec);
+            break;
+    }
+}
+
+static void
+xfce_backdrop_get_property(GObject *object,
+                           guint property_id,
+                           GValue *value,
+                           GParamSpec *pspec)
+{
+    XfceBackdrop *backdrop = XFCE_BACKDROP(object);
+
+    switch(property_id) {
+        case PROP_COLOR_STYLE:
+            g_value_set_enum(value, xfce_backdrop_get_color_style(backdrop));
+            break;
+
+        case PROP_COLOR1:
+            g_value_set_boxed(value, &backdrop->priv->color1);
+            break;
+
+        case PROP_COLOR2:
+            g_value_set_boxed(value, &backdrop->priv->color2);
+            break;
+
+        case PROP_SHOW_IMAGE:
+            g_value_set_boolean(value, xfce_backdrop_get_show_image(backdrop));
+            break;
+
+        case PROP_IMAGE_STYLE:
+            g_value_set_enum(value, xfce_backdrop_get_image_style(backdrop));
+            break;
+
+        case PROP_IMAGE_FILENAME:
+            g_value_set_string(value,
+                               xfce_backdrop_get_image_filename(backdrop));
+            break;
+
+        case PROP_BRIGHTNESS:
+            g_value_set_int(value, xfce_backdrop_get_brightness(backdrop));
+            break;
+
+        case PROP_SATURATION:
+            g_value_set_double(value, xfce_backdrop_get_saturation(backdrop));
+            break;
+
+        default:
+            G_OBJECT_WARN_INVALID_PROPERTY_ID(object, property_id, pspec);
+            break;
+    }
+}
+
+
 
 /* public api */
 
@@ -263,7 +474,9 @@ xfce_backdrop_new(GdkVisual *visual)
  * Return value: A new #XfceBackdrop.
  **/
 XfceBackdrop *
-xfce_backdrop_new_with_size(GdkVisual *visual, gint width, gint height)
+xfce_backdrop_new_with_size(GdkVisual *visual,
+                            gint width,
+                            gint height)
 {
     XfceBackdrop *backdrop;
     
@@ -274,7 +487,7 @@ xfce_backdrop_new_with_size(GdkVisual *visual, gint width, gint height)
     backdrop->priv->bpp = visual->depth;
     backdrop->priv->width = width;
     backdrop->priv->height = height;
-    
+
     return backdrop;
 }
 
@@ -309,6 +522,7 @@ xfce_backdrop_set_color_style(XfceBackdrop *backdrop,
         XfceBackdropColorStyle style)
 {
     g_return_if_fail(XFCE_IS_BACKDROP(backdrop));
+    g_return_if_fail((int)style >= 0 && style <= XFCE_BACKDROP_COLOR_TRANSPARENT);
     
     if(style != backdrop->priv->color_style) {
         backdrop->priv->color_style = style;
@@ -469,10 +683,12 @@ xfce_backdrop_set_image_filename(XfceBackdrop *backdrop, const gchar *filename)
 {
     g_return_if_fail(XFCE_IS_BACKDROP(backdrop));
     
-    if(backdrop->priv->image_path)
-        g_free(backdrop->priv->image_path);
+    g_free(backdrop->priv->image_path);
     
-    backdrop->priv->image_path = g_strdup(filename);
+    if(filename)
+        backdrop->priv->image_path = g_strdup(filename);
+    else
+        backdrop->priv->image_path = NULL;
     
     g_signal_emit(G_OBJECT(backdrop), backdrop_signals[BACKDROP_CHANGED], 0);
 }
@@ -511,6 +727,24 @@ xfce_backdrop_get_brightness(XfceBackdrop *backdrop)
     return backdrop->priv->brightness;
 }
 
+void
+xfce_backdrop_set_saturation(XfceBackdrop *backdrop,
+                             gdouble saturation)
+{
+    g_return_if_fail(XFCE_IS_BACKDROP(backdrop));
+    if(abs(saturation - backdrop->priv->saturation) > 0.05) {
+        backdrop->priv->saturation = saturation;
+        g_signal_emit(G_OBJECT(backdrop), backdrop_signals[BACKDROP_CHANGED], 0);
+    }
+}
+
+gdouble
+xfce_backdrop_get_saturation(XfceBackdrop *backdrop)
+{
+    g_return_val_if_fail(XFCE_IS_BACKDROP(backdrop), 1.0);
+    return backdrop->priv->saturation;
+}
+
 /**
  * xfce_backdrop_get_pixbuf:
  * @backdrop: An #XfceBackdrop.
@@ -552,12 +786,15 @@ xfce_backdrop_get_pixbuf(XfceBackdrop *backdrop)
     }
     
     if(backdrop->priv->color_style == XFCE_BACKDROP_COLOR_SOLID)
-        final_image = create_solid(&backdrop->priv->color1, w, h);
-    else {
+        final_image = create_solid(&backdrop->priv->color1, w, h, FALSE, 0xff);
+    else if(backdrop->priv->color_style == XFCE_BACKDROP_COLOR_TRANSPARENT) {
+        GdkColor c = { 0, 0xffff, 0xffff, 0xffff };
+        final_image = create_solid(&c, w, h, TRUE, 0x00);
+    } else {
         final_image = create_gradient(&backdrop->priv->color1,
                 &backdrop->priv->color2, w, h, backdrop->priv->color_style);
         if(!final_image)
-            final_image = create_solid(&backdrop->priv->color1, w, h);
+            final_image = create_solid(&backdrop->priv->color1, w, h, FALSE, 0xff);
     }
     
     if(!image) {
@@ -571,7 +808,7 @@ xfce_backdrop_get_pixbuf(XfceBackdrop *backdrop)
         if(ih <= h / 2 && iw <= w / 2)
             istyle = XFCE_BACKDROP_IMAGE_TILED;
         else
-            istyle = XFCE_BACKDROP_IMAGE_SCALED;
+            istyle = XFCE_BACKDROP_IMAGE_ZOOMED;
     } else
         istyle = backdrop->priv->image_style;
     
@@ -656,6 +893,22 @@ xfce_backdrop_get_pixbuf(XfceBackdrop *backdrop)
                     interp, 255);
             break;
         
+        case XFCE_BACKDROP_IMAGE_ZOOMED:
+            xscale = (gdouble)w / iw;
+            yscale = (gdouble)h / ih;
+            if(xscale < yscale) {
+                xscale = yscale;
+                xo = (w - (iw * xscale)) * 0.5;
+                yo = 0;
+            } else {
+                yscale = xscale;
+                xo = 0;
+                yo = (h - (ih * yscale)) * 0.5;
+            }
+            gdk_pixbuf_composite(image, final_image, 0, 0,
+                    w, h, xo, yo, xscale, yscale, interp, 255);
+            break;
+        
         default:
             g_critical("Invalid image style: %d\n", (gint)istyle);
     }
@@ -665,6 +918,12 @@ xfce_backdrop_get_pixbuf(XfceBackdrop *backdrop)
     
     if(backdrop->priv->brightness != 0)
         final_image = adjust_brightness(final_image, backdrop->priv->brightness);
+
+    if(backdrop->priv->saturation > 1.01 || backdrop->priv->saturation < 0.99) {
+        gdk_pixbuf_saturate_and_pixelate(final_image, final_image,
+                                         (gfloat)backdrop->priv->saturation,
+                                         FALSE);
+    }
     
     return final_image;
 }
