@@ -34,10 +34,13 @@
 
 #include <thunar-vfs/thunar-vfs.h>
 
+#include <dbus/dbus-glib-lowlevel.h>
+
 #ifdef HAVE_THUNARX
 #include <thunarx/thunarx.h>
 #endif
 
+#include "xfdesktop-dbus-bindings-filemanager.h"
 #include "xfdesktop-file-icon.h"
 #include "xfdesktop-file-utils.h"
 
@@ -230,14 +233,15 @@ xfdesktop_file_utils_get_file_icon(const gchar *custom_icon_name,
         if(icon_name)
             pix_theme = xfce_themed_icon_load(icon_name, size);
     }
-            
-    if(G_UNLIKELY(!pix_theme) && info) {
+
+    if(!pix_theme && info && info->mime_info) {
         icon_name = thunar_vfs_mime_info_lookup_icon_name(info->mime_info,
                                                           gtk_icon_theme_get_default());
+        DBG("got mime info icon name: %s", icon_name);
         if(icon_name)
             pix_theme = xfce_themed_icon_load(icon_name, size);
     }
-    
+
     if(G_LIKELY(pix_theme)) {
         /* we can't edit thsese icons */
         pix = gdk_pixbuf_copy(pix_theme);
@@ -250,7 +254,10 @@ xfdesktop_file_utils_get_file_icon(const gchar *custom_icon_name,
         pix = xfdesktop_file_utils_get_fallback_icon(size);
     
     /* sanity check */
-    g_return_val_if_fail(G_LIKELY(pix), NULL);
+    if(G_UNLIKELY(!pix)) {
+        g_warning("Unable to find fallback icon");
+        return NULL;
+    }
     
     if(emblem) {
         gint emblem_pix_size = gdk_pixbuf_get_width(emblem);
@@ -293,55 +300,205 @@ xfdesktop_file_utils_get_file_icon(const gchar *custom_icon_name,
     return pix;
 }
 
+void
+xfdesktop_file_utils_set_window_cursor(GtkWindow *window,
+                                       GdkCursorType cursor_type)
+{
+    GdkCursor *cursor;
+    
+    if(!window || !GTK_WIDGET(window)->window)
+        return;
+    
+    cursor = gdk_cursor_new(cursor_type);
+    if(G_LIKELY(cursor)) {
+        gdk_window_set_cursor(GTK_WIDGET(window)->window, cursor);
+        gdk_cursor_unref(cursor);
+    }
+}
+
 gboolean
-xfdesktop_file_utils_launch_external(const ThunarVfsInfo *info,
-                                     GdkScreen *screen)
+xfdesktop_file_utils_launch_fallback(const ThunarVfsInfo *info,
+                                     GdkScreen *screen,
+                                     GtkWindow *parent)
 {
     gboolean ret = FALSE;
-    gchar *folder_name, *display_name, *commandline;
-    gint status = 0;
+    gchar *file_manager_app;
     
     g_return_val_if_fail(info, FALSE);
     
-    if(!screen)
-        screen = gdk_display_get_default_screen(gdk_display_get_default());
-    
-    folder_name = thunar_vfs_path_dup_uri(info->path);
-    display_name = gdk_screen_make_display_name(screen);
-    
-    /* try the org.xfce.FileManager D-BUS interface first */
-    commandline = g_strdup_printf("dbus-send --print-reply --dest=org.xfce.FileManager "
-                                  "/org/xfce/FileManager org.xfce.FileManager.Launch "
-                                  "string:\"%s\" string:\"%s\"",
-                                  folder_name, display_name);
-    
-    ret = (g_spawn_command_line_sync(commandline, NULL, NULL, &status, NULL)
-           && status == 0);
-    g_free(commandline);
-
-    /* hardcoded fallback to a file manager if that didn't work */
-    if(!ret) {
-        gchar *file_manager_app = g_find_program_in_path(FILE_MANAGER_FALLBACK);
+    file_manager_app = g_find_program_in_path(FILE_MANAGER_FALLBACK);
+    if(file_manager_app) {
+        gchar *commandline, *uri, *display_name;
         
-        if(file_manager_app) {
-            commandline = g_strconcat("env DISPLAY=\"", display_name, "\" \"",
-                                      file_manager_app, "\" \"", folder_name,
-                                      "\"", NULL);
-            
-            DBG("executing:\n%s\n", commandline);
-            
-            ret = xfce_exec(commandline, FALSE, TRUE, NULL);
-            g_free(commandline);
-            g_free(file_manager_app);
-        }
+        if(!screen && parent)
+            screen = gtk_widget_get_screen(GTK_WIDGET(parent));
+        else if(!screen)
+            screen = gdk_display_get_default_screen(gdk_display_get_default());
+        
+        display_name = gdk_screen_make_display_name(screen);
+        uri = thunar_vfs_path_dup_uri(info->path);
+        
+        commandline = g_strconcat("env DISPLAY=\"", display_name,
+                                  "\" \"", file_manager_app, "\" \"",
+                                  uri, "\"", NULL);
+        
+        DBG("executing:\n%s\n", commandline);
+        
+        ret = xfce_exec(commandline, FALSE, TRUE, NULL);
+        
+        g_free(commandline);
+        g_free(file_manager_app);
+        g_free(uri);
+        g_free(display_name);
+    }
+    
+    if(!ret) {
+        gchar *primary = g_markup_printf_escaped(_("Unable to launch \"%s\":"),
+                                                 info->display_name);
+        xfce_message_dialog(GTK_WINDOW(parent),
+                            _("Launch Error"), GTK_STOCK_DIALOG_ERROR,
+                            primary,
+                            _("This feature requires a file manager service present (such as that supplied by Thunar)."),
+                            GTK_STOCK_CLOSE, GTK_RESPONSE_ACCEPT, NULL);
+        g_free(primary);
     }
-
-    g_free(display_name);
-    g_free(folder_name);
     
     return ret;
 }
 
+typedef struct
+{
+    const ThunarVfsInfo *info;
+    GdkScreen *screen;
+    GtkWindow *parent;
+} XfdesktopDisplayFolderData;
+
+static void
+xfdesktop_file_utils_display_folder_cb(DBusGProxy *proxy,
+                                       GError *error,
+                                       gpointer user_data)
+{
+    XfdesktopDisplayFolderData *dfdata = user_data;
+    
+    g_return_if_fail(dfdata);
+    
+    xfdesktop_file_utils_set_window_cursor(dfdata->parent, GDK_LEFT_PTR);
+    
+    if(error) {
+        xfdesktop_file_utils_launch_fallback(dfdata->info, dfdata->screen,
+                                             dfdata->parent);
+    }
+    
+    g_free(user_data);
+}
+
+void
+xfdesktop_file_utils_open_folder(const ThunarVfsInfo *info,
+                                 GdkScreen *screen,
+                                 GtkWindow *parent)
+{
+    DBusGProxy *fileman_proxy;
+    
+    g_return_if_fail(info && (screen || parent));
+    
+    if(!screen)
+        screen = gtk_widget_get_screen(GTK_WIDGET(parent));
+    
+    fileman_proxy = xfdesktop_file_utils_peek_filemanager_proxy();
+    if(fileman_proxy) {
+        XfdesktopDisplayFolderData *dfdata = g_new(XfdesktopDisplayFolderData, 1);
+        gchar *uri = thunar_vfs_path_dup_uri(info->path);
+        gchar *display_name = gdk_screen_make_display_name(screen);
+        
+        dfdata->info = info;
+        dfdata->screen = screen;
+        dfdata->parent = parent;
+        if(!org_xfce_FileManager_display_folder_async(fileman_proxy,
+                                                      uri, display_name,
+                                                      xfdesktop_file_utils_display_folder_cb,
+                                                      dfdata))
+        {
+            xfdesktop_file_utils_launch_fallback(info, screen, parent);
+            g_free(dfdata);
+        } else
+            xfdesktop_file_utils_set_window_cursor(parent, GDK_WATCH);
+        
+        g_free(uri);
+        g_free(display_name);
+    } else
+        xfdesktop_file_utils_launch_fallback(info, screen, parent);
+}
+
+
+static gint dbus_ref_cnt = 0;
+static DBusGConnection *dbus_gconn = NULL;
+static DBusGProxy *dbus_trash_proxy = NULL;
+static DBusGProxy *dbus_filemanager_proxy = NULL;
+
+gboolean
+xfdesktop_file_utils_dbus_init(void)
+{
+    gboolean ret = TRUE;
+    
+    if(dbus_ref_cnt++)
+        return TRUE;
+    
+    if(!dbus_gconn) {
+        dbus_gconn = dbus_g_bus_get(DBUS_BUS_SESSION, NULL);
+        if(G_LIKELY(dbus_gconn)) {
+            /* dbus's default is brain-dead */
+            DBusConnection *dconn = dbus_g_connection_get_connection(dbus_gconn);
+            dbus_connection_set_exit_on_disconnect(dconn, FALSE);
+        }
+    }
+    
+    if(G_LIKELY(dbus_gconn)) {
+        dbus_trash_proxy = dbus_g_proxy_new_for_name(dbus_gconn,
+                                                     "org.xfce.FileManager",
+                                                     "/org/xfce/FileManager",
+                                                     "org.xfce.Trash");
+        dbus_g_proxy_add_signal(dbus_trash_proxy, "TrashChanged",
+                                G_TYPE_BOOLEAN, G_TYPE_INVALID);
+        
+        dbus_filemanager_proxy = dbus_g_proxy_new_for_name(dbus_gconn,
+                                                           "org.xfce.FileManager",
+                                                           "/org/xfce/FileManager",
+                                                           "org.xfce.FileManager");
+    } else {
+        ret = FALSE;
+        dbus_ref_cnt = 0;
+    }
+    
+    return ret;
+}
+
+DBusGProxy *
+xfdesktop_file_utils_peek_trash_proxy(void)
+{
+    return dbus_trash_proxy;
+}
+
+DBusGProxy *
+xfdesktop_file_utils_peek_filemanager_proxy(void)
+{
+    return dbus_filemanager_proxy;
+}
+
+void
+xfdesktop_file_utils_dbus_cleanup(void)
+{
+    if(dbus_ref_cnt == 0 || --dbus_ref_cnt > 0)
+        return;
+    
+    if(dbus_trash_proxy)
+        g_object_unref(G_OBJECT(dbus_trash_proxy));
+    if(dbus_filemanager_proxy)
+        g_object_unref(G_OBJECT(dbus_filemanager_proxy));
+    
+    /* we aren't going to unref dbus_gconn because dbus appears to have a
+     * memleak in dbus_connection_setup_with_g_main().  really; the comments
+     * in dbus-gmain.c admit this. */
+}
 
 
 
@@ -371,9 +528,8 @@ xfdesktop_thunarx_file_info_get_uri(ThunarxFileInfo *file_info)
     if(!info)
         return NULL;
         
-    g_return_val_if_fail(thunar_vfs_path_to_uri(info->path, buf, PATH_MAX,
-                                                NULL) > 0,
-                         NULL);
+    if(thunar_vfs_path_to_uri(info->path, buf, PATH_MAX, NULL) <= 0)
+        return NULL;
     
     return g_strdup(buf);
 }
@@ -394,9 +550,8 @@ xfdesktop_thunarx_file_info_get_parent_uri(ThunarxFileInfo *file_info)
     if(G_UNLIKELY(!parent))
         return NULL;
     
-    g_return_val_if_fail(thunar_vfs_path_to_uri(parent, buf, PATH_MAX,
-                                                NULL) > 0,
-                         NULL);
+    if(thunar_vfs_path_to_uri(parent, buf, PATH_MAX, NULL) <= 0)
+        return NULL;
     
     return g_strdup(buf);
 }
