@@ -79,7 +79,7 @@ static gboolean xfdesktop_regular_file_icon_do_drop_dest(XfdesktopIcon *icon,
 static G_CONST_RETURN ThunarVfsInfo *xfdesktop_regular_file_icon_peek_info(XfdesktopFileIcon *icon);
 static void xfdesktop_regular_file_icon_update_info(XfdesktopFileIcon *icon,
                                                     ThunarVfsInfo *info);
-static gboolean xfdesktop_regular_file_can_write_file(XfdesktopFileIcon *icon);
+static gboolean xfdesktop_regular_file_can_write_parent(XfdesktopFileIcon *icon);
 static gboolean xfdesktop_regular_file_icon_rename_file(XfdesktopFileIcon *icon,
                                                         const gchar *new_name);
 static gboolean xfdesktop_regular_file_icon_delete_file(XfdesktopFileIcon *icon);
@@ -127,9 +127,9 @@ xfdesktop_regular_file_icon_class_init(XfdesktopRegularFileIconClass *klass)
     
     file_icon_class->peek_info = xfdesktop_regular_file_icon_peek_info;
     file_icon_class->update_info = xfdesktop_regular_file_icon_update_info;
-    file_icon_class->can_rename_file = xfdesktop_regular_file_can_write_file;
+    file_icon_class->can_rename_file = xfdesktop_regular_file_can_write_parent;
     file_icon_class->rename_file = xfdesktop_regular_file_icon_rename_file;
-    file_icon_class->can_delete_file = xfdesktop_regular_file_can_write_file;
+    file_icon_class->can_delete_file = xfdesktop_regular_file_can_write_parent;
     file_icon_class->delete_file = xfdesktop_regular_file_icon_delete_file;
 }
 
@@ -254,7 +254,8 @@ xfdesktop_regular_file_icon_get_allowed_drag_actions(XfdesktopIcon *icon)
     const ThunarVfsInfo *info = xfdesktop_file_icon_peek_info(XFDESKTOP_FILE_ICON(icon));
     GdkDragAction actions = GDK_ACTION_LINK;  /* we can always link */
     
-    g_return_val_if_fail(info, 0);
+    if(!info)
+        return 0;
     
     if(info->flags & THUNAR_VFS_FILE_FLAGS_READABLE) {
         ThunarVfsPath *parent_path;
@@ -282,7 +283,8 @@ xfdesktop_regular_file_icon_get_allowed_drop_actions(XfdesktopIcon *icon)
 {
     const ThunarVfsInfo *info = xfdesktop_file_icon_peek_info(XFDESKTOP_FILE_ICON(icon));
     
-    g_return_val_if_fail(info, 0);
+    if(!info)
+        return 0;
     
     /* if it's executable we can 'copy'.  if it's a folder we can do anything
      * if it's writable. */
@@ -307,7 +309,10 @@ xfdesktop_regular_file_icon_drag_job_error(ThunarVfsJob *job,
     XfdesktopFileUtilsFileop fileop = GPOINTER_TO_INT(g_object_get_data(G_OBJECT(job),
                                                                         "--xfdesktop-file-icon-action"));
     
-    g_return_if_fail(regular_file_icon && src_file_icon);
+    g_return_if_fail(regular_file_icon);
+    
+    if(!src_file_icon)
+        return;
     
     xfdesktop_file_utils_handle_fileop_error(NULL,
                                              xfdesktop_file_icon_peek_info(src_file_icon),
@@ -400,11 +405,13 @@ xfdesktop_regular_file_icon_do_drop_dest(XfdesktopIcon *icon,
             return FALSE;
         
         name = thunar_vfs_path_get_name(src_info->path);
-        g_return_val_if_fail(name, FALSE);
+        if(!name)
+            return FALSE;
         
         dest_path = thunar_vfs_path_relative(regular_file_icon->priv->info->path,
                                              name);
-        g_return_val_if_fail(dest_path, FALSE);
+        if(!dest_path)
+            return FALSE;
         
         switch(action) {
             case GDK_ACTION_MOVE:
@@ -497,13 +504,28 @@ xfdesktop_regular_file_icon_peek_tooltip(XfdesktopIcon *icon)
 }
 
 static gboolean
-xfdesktop_regular_file_can_write_file(XfdesktopFileIcon *icon)
+xfdesktop_regular_file_can_write_parent(XfdesktopFileIcon *icon)
 {
     XfdesktopRegularFileIcon *file_icon = XFDESKTOP_REGULAR_FILE_ICON(icon);
-    
+    ThunarVfsPath *parent;
+    ThunarVfsInfo *parent_info;
+    gboolean writable;
+
     g_return_val_if_fail(file_icon && file_icon->priv->info, FALSE);
-    
-    return (file_icon->priv->info->flags & THUNAR_VFS_FILE_FLAGS_WRITABLE);
+
+    parent = thunar_vfs_path_get_parent(file_icon->priv->info->path);
+    if(!parent)
+        return FALSE;
+
+    parent_info = thunar_vfs_info_new_for_path(parent, NULL);
+    if(!parent_info)
+        return FALSE;
+
+    writable = (parent_info->flags & THUNAR_VFS_FILE_FLAGS_WRITABLE);
+    thunar_vfs_info_unref(parent_info);
+
+    return writable;
+
 }
 
 static void
