@@ -77,7 +77,8 @@ static GdkDragAction xfdesktop_special_file_icon_get_allowed_drop_actions(Xfdesk
 static gboolean xfdesktop_special_file_icon_do_drop_dest(XfdesktopIcon *icon,
                                                          XfdesktopIcon *src_icon,
                                                          GdkDragAction action);
-static GtkWidget *xfdesktop_special_file_icon_get_popup_menu(XfdesktopIcon *icon);
+static gboolean xfdesktop_special_file_icon_populate_context_menu(XfdesktopIcon *icon,
+                                                                  GtkWidget *menu);
 
 static G_CONST_RETURN ThunarVfsInfo *xfdesktop_special_file_icon_peek_info(XfdesktopFileIcon *icon);
 
@@ -122,7 +123,7 @@ xfdesktop_special_file_icon_class_init(XfdesktopSpecialFileIconClass *klass)
     icon_class->get_allowed_drag_actions = xfdesktop_special_file_icon_get_allowed_drag_actions;
     icon_class->get_allowed_drop_actions = xfdesktop_special_file_icon_get_allowed_drop_actions;
     icon_class->do_drop_dest = xfdesktop_special_file_icon_do_drop_dest;
-    icon_class->get_popup_menu = xfdesktop_special_file_icon_get_popup_menu;
+    icon_class->populate_context_menu = xfdesktop_special_file_icon_populate_context_menu;
     
     file_icon_class->peek_info = xfdesktop_special_file_icon_peek_info;
     file_icon_class->can_rename_file = (gboolean (*)(XfdesktopFileIcon *))gtk_false;
@@ -332,7 +333,10 @@ xfdesktop_special_file_icon_drag_job_error(ThunarVfsJob *job,
                                                                         "--xfdesktop-file-icon-action"));
     const ThunarVfsInfo *src_info = xfdesktop_file_icon_peek_info(src_file_icon);
     
-    g_return_if_fail(special_file_icon && src_file_icon);
+    g_return_if_fail(special_file_icon);
+
+    if(!src_file_icon)
+        return;
     
     xfdesktop_file_utils_handle_fileop_error(NULL, src_info,
                                              special_file_icon->priv->info,
@@ -385,8 +389,7 @@ xfdesktop_special_file_icon_do_drop_dest(XfdesktopIcon *icon,
     
     DBG("entering");
     
-    g_return_val_if_fail(special_file_icon && src_file_icon,
-                         FALSE);
+    g_return_val_if_fail(special_file_icon && src_file_icon, FALSE);
     g_return_val_if_fail(xfdesktop_special_file_icon_get_allowed_drop_actions(icon),
                          FALSE);
     
@@ -398,11 +401,13 @@ xfdesktop_special_file_icon_do_drop_dest(XfdesktopIcon *icon,
         return FALSE;
     
     name = thunar_vfs_path_get_name(src_info->path);
-    g_return_val_if_fail(name, FALSE);
+    if(!name)
+        return FALSE;
     
     dest_path = thunar_vfs_path_relative(special_file_icon->priv->info->path,
                                          name);
-    g_return_val_if_fail(dest_path, FALSE);
+    if(!dest_path)
+        return FALSE;
     
     if(special_file_icon->priv->type == XFDESKTOP_SPECIAL_FILE_ICON_TRASH)  {
         /* any drop to the trash is a move */
@@ -475,23 +480,29 @@ xfdesktop_special_file_icon_peek_tooltip(XfdesktopIcon *icon)
 {
     XfdesktopSpecialFileIcon *special_file_icon = XFDESKTOP_SPECIAL_FILE_ICON(icon);
     
-    /* FIXME: implement trash stuff */
-    
     if(!special_file_icon->priv->tooltip) {
-        gchar mod[64], *kind, sizebuf[64], *size;
-        struct tm *tm = localtime(&special_file_icon->priv->info->mtime);
+        if(XFDESKTOP_SPECIAL_FILE_ICON_TRASH == special_file_icon->priv->type) {
+            /* FIXME: also display # of items in trash */
+            special_file_icon->priv->tooltip = g_strdup(_("Kind: Trash"));
+        } else {
+            gchar mod[64], *kind, sizebuf[64], *size;
+            struct tm *tm = localtime(&special_file_icon->priv->info->mtime);
 
-        strftime(mod, 64, "%Y-%m-%d %H:%M:%S", tm);
-        kind = xfdesktop_file_utils_get_file_kind(special_file_icon->priv->info, NULL);
-        thunar_vfs_humanize_size(special_file_icon->priv->info->size, sizebuf, 64);
-        size = g_strdup_printf(_("%s (%" G_GINT64_FORMAT " Bytes)"), sizebuf,
-                              (gint64)special_file_icon->priv->info->size);
-        
-        special_file_icon->priv->tooltip = g_strdup_printf(_("Kind: %s\nModified:%s\nSize: %s"),
-                                                   kind, mod, size);
-        
-        g_free(kind);
-        g_free(size);
+            strftime(mod, 64, "%Y-%m-%d %H:%M:%S", tm);
+            kind = xfdesktop_file_utils_get_file_kind(special_file_icon->priv->info,
+                                                      NULL);
+            thunar_vfs_humanize_size(special_file_icon->priv->info->size,
+                                     sizebuf, 64);
+            size = g_strdup_printf(_("%s (%" G_GINT64_FORMAT " Bytes)"),
+                                   sizebuf,
+                                  (gint64)special_file_icon->priv->info->size);
+            
+            special_file_icon->priv->tooltip = g_strdup_printf(_("Kind: %s\nModified:%s\nSize: %s"),
+                                                               kind, mod, size);
+            
+            g_free(kind);
+            g_free(size);
+        }
     }
     
     return special_file_icon->priv->tooltip;
@@ -608,16 +619,18 @@ xfdesktop_special_file_icon_trash_empty(GtkWidget *w,
     }
 }
 
-static GtkWidget *
-xfdesktop_special_file_icon_get_popup_menu(XfdesktopIcon *icon)
+static gboolean
+xfdesktop_special_file_icon_populate_context_menu(XfdesktopIcon *icon,
+                                                  GtkWidget *menu)
 {
     XfdesktopSpecialFileIcon *special_file_icon = XFDESKTOP_SPECIAL_FILE_ICON(icon);
-    GtkWidget *menu, *mi, *img;
+    GtkWidget *mi, *img;
+    GtkIconTheme *icon_theme;
     
     if(XFDESKTOP_SPECIAL_FILE_ICON_TRASH != special_file_icon->priv->type)
-        return NULL;
+        return FALSE;
     
-    menu = gtk_menu_new();
+    icon_theme = gtk_icon_theme_get_default();
     
     img = gtk_image_new_from_stock(GTK_STOCK_OPEN, GTK_ICON_SIZE_MENU);
     gtk_widget_show(img);
@@ -632,7 +645,16 @@ xfdesktop_special_file_icon_get_popup_menu(XfdesktopIcon *icon)
     gtk_widget_show(mi);
     gtk_menu_shell_append(GTK_MENU_SHELL(menu), mi);
     
+    if(gtk_icon_theme_has_icon(icon_theme, "user-trash"))
+        img = gtk_image_new_from_icon_name("user-trash", GTK_ICON_SIZE_MENU);
+    else if(gtk_icon_theme_has_icon(icon_theme, "gnome-fs-trash-empty"))
+        img = gtk_image_new_from_icon_name("gnome-fs-trash-empty", GTK_ICON_SIZE_MENU);
+    else
+        img = NULL;
+    
     mi = gtk_image_menu_item_new_with_mnemonic(_("_Empty Trash"));
+    if(img)
+        gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(mi), img);
     gtk_widget_show(mi);
     gtk_menu_shell_append(GTK_MENU_SHELL(menu), mi);
     if(special_file_icon->priv->trash_full) {
@@ -642,7 +664,7 @@ xfdesktop_special_file_icon_get_popup_menu(XfdesktopIcon *icon)
     } else
         gtk_widget_set_sensitive(mi, FALSE);
     
-    return menu;
+    return TRUE;
 }
 
 
