@@ -35,6 +35,10 @@
 #include <time.h>
 #endif
 
+#ifdef HAVE_SYS_STATVFS_H
+#include <sys/statvfs.h>
+#endif
+
 #ifndef PATH_MAX
 #define PATH_MAX 4096
 #endif
@@ -46,8 +50,6 @@
 #endif
 
 #include "xfdesktop-file-utils.h"
-#include "xfdesktop-icon.h"
-#include "xfdesktop-file-icon.h"
 #include "xfdesktop-file-properties-dialog.h"
 #include "xfdesktop-volume-icon.h"
 
@@ -63,42 +65,41 @@ struct _XfdesktopVolumeIconPrivate
 
 static void xfdesktop_volume_icon_finalize(GObject *obj);
 
-static void xfdesktop_volume_icon_icon_init(XfdesktopIconIface *iface);
 static GdkPixbuf *xfdesktop_volume_icon_peek_pixbuf(XfdesktopIcon *icon,
                                                     gint size);
 static G_CONST_RETURN gchar *xfdesktop_volume_icon_peek_label(XfdesktopIcon *icon);
 static G_CONST_RETURN gchar *xfdesktop_volume_icon_peek_tooltip(XfdesktopIcon *icon);
-static gboolean xfdesktop_volume_icon_is_drop_dest(XfdesktopIcon *icon);
-static XfdesktopIconDragResult xfdesktop_volume_icon_do_drop_dest(XfdesktopIcon *icon,
-                                                                  XfdesktopIcon *src_icon,
-                                                                  GdkDragAction action);
-static GtkWidget *xfdesktop_volume_icon_get_popup_menu(XfdesktopIcon *icon);
+static GdkDragAction xfdesktop_volume_icon_get_allowed_drag_actions(XfdesktopIcon *icon);
+static GdkDragAction xfdesktop_volume_icon_get_allowed_drop_actions(XfdesktopIcon *icon);
+static gboolean xfdesktop_volume_icon_do_drop_dest(XfdesktopIcon *icon,
+                                                   XfdesktopIcon *src_icon,
+                                                   GdkDragAction action);
+static gboolean xfdesktop_volume_icon_populate_context_menu(XfdesktopIcon *icon,
+                                                            GtkWidget *menu);
 
 static G_CONST_RETURN ThunarVfsInfo *xfdesktop_volume_icon_peek_info(XfdesktopFileIcon *icon);
 static void xfdesktop_volume_icon_update_info(XfdesktopFileIcon *icon,
                                               ThunarVfsInfo *info);
+static gboolean xfdesktop_volume_icon_activated(XfdesktopIcon *icon);
 
 #ifdef HAVE_THUNARX
 static void xfdesktop_volume_icon_tfi_init(ThunarxFileInfoIface *iface);
 #endif
 
+static void xfdesktop_volume_icon_volume_changed_cb(ThunarVfsVolume *volume,
+                                                    gpointer user_data);
 static inline void xfdesktop_volume_icon_invalidate_pixbuf(XfdesktopVolumeIcon *icon);
 
 
 #ifdef HAVE_THUNARX
 G_DEFINE_TYPE_EXTENDED(XfdesktopVolumeIcon, xfdesktop_volume_icon,
                        XFDESKTOP_TYPE_FILE_ICON, 0,
-                       G_IMPLEMENT_INTERFACE(XFDESKTOP_TYPE_ICON,
-                                             xfdesktop_volume_icon_icon_init)
                        G_IMPLEMENT_INTERFACE(THUNARX_TYPE_FILE_INFO,
                                              xfdesktop_volume_icon_tfi_init)
                        )
 #else
-G_DEFINE_TYPE_EXTENDED(XfdesktopVolumeIcon, xfdesktop_volume_icon,
-                       XFDESKTOP_TYPE_FILE_ICON, 0,
-                       G_IMPLEMENT_INTERFACE(XFDESKTOP_TYPE_ICON,
-                                             xfdesktop_volume_icon_icon_init)
-                       )
+G_DEFINE_TYPE(XfdesktopVolumeIcon, xfdesktop_volume_icon,
+              XFDESKTOP_TYPE_FILE_ICON)
 #endif
 
 
@@ -107,12 +108,22 @@ static void
 xfdesktop_volume_icon_class_init(XfdesktopVolumeIconClass *klass)
 {
     GObjectClass *gobject_class = (GObjectClass *)klass;
+    XfdesktopIconClass *icon_class = (XfdesktopIconClass *)klass;
     XfdesktopFileIconClass *file_icon_class = (XfdesktopFileIconClass *)klass;
     
     g_type_class_add_private(klass, sizeof(XfdesktopVolumeIconClass));
     
     gobject_class->finalize = xfdesktop_volume_icon_finalize;
     
+    icon_class->peek_pixbuf = xfdesktop_volume_icon_peek_pixbuf;
+    icon_class->peek_label = xfdesktop_volume_icon_peek_label;
+    icon_class->peek_tooltip = xfdesktop_volume_icon_peek_tooltip;
+    icon_class->get_allowed_drag_actions = xfdesktop_volume_icon_get_allowed_drag_actions;
+    icon_class->get_allowed_drop_actions = xfdesktop_volume_icon_get_allowed_drop_actions;
+    icon_class->do_drop_dest = xfdesktop_volume_icon_do_drop_dest;
+    icon_class->populate_context_menu = xfdesktop_volume_icon_populate_context_menu;
+    icon_class->activated = xfdesktop_volume_icon_activated;
+    
     file_icon_class->peek_info = xfdesktop_volume_icon_peek_info;
     file_icon_class->update_info = xfdesktop_volume_icon_update_info;
     file_icon_class->can_rename_file = (gboolean (*)(XfdesktopFileIcon *))gtk_false;
@@ -136,6 +147,10 @@ xfdesktop_volume_icon_finalize(GObject *obj)
                                          G_CALLBACK(xfdesktop_volume_icon_invalidate_pixbuf),
                                          icon);
     
+    g_signal_handlers_disconnect_by_func(G_OBJECT(icon->priv->volume),
+                                         G_CALLBACK(xfdesktop_volume_icon_volume_changed_cb),
+                                         icon);
+    
     if(icon->priv->pix)
         g_object_unref(G_OBJECT(icon->priv->pix));
     
@@ -151,17 +166,6 @@ xfdesktop_volume_icon_finalize(GObject *obj)
     G_OBJECT_CLASS(xfdesktop_volume_icon_parent_class)->finalize(obj);
 }
 
-static void
-xfdesktop_volume_icon_icon_init(XfdesktopIconIface *iface)
-{
-    iface->peek_pixbuf = xfdesktop_volume_icon_peek_pixbuf;
-    iface->peek_label = xfdesktop_volume_icon_peek_label;
-    iface->peek_tooltip = xfdesktop_volume_icon_peek_tooltip;
-    iface->is_drop_dest = xfdesktop_volume_icon_is_drop_dest;
-    iface->do_drop_dest = xfdesktop_volume_icon_do_drop_dest;
-    iface->get_popup_menu = xfdesktop_volume_icon_get_popup_menu;
-}
-
 #ifdef HAVE_THUNARX
 static void
 xfdesktop_volume_icon_tfi_init(ThunarxFileInfoIface *iface)
@@ -177,6 +181,31 @@ xfdesktop_volume_icon_tfi_init(ThunarxFileInfoIface *iface)
 }
 #endif  /* HAVE_THUNARX */
 
+
+static void
+xfdesktop_volume_icon_volume_changed_cb(ThunarVfsVolume *volume,
+                                        gpointer user_data)
+{
+    XfdesktopFileIcon *file_icon = XFDESKTOP_FILE_ICON(user_data);
+    ThunarVfsPath *new_path;
+    
+    new_path = thunar_vfs_volume_get_mount_point(volume);
+    if(new_path) {
+        ThunarVfsInfo *new_info = thunar_vfs_info_new_for_path(new_path, NULL);
+        
+        if(new_info) {
+            xfdesktop_file_icon_update_info(file_icon, new_info);
+            thunar_vfs_info_unref(new_info);
+            return;
+        }
+        
+        /* |new_path| is owned by |volume| */
+    }
+    
+    /* both new and old info is NULL or stale */
+    xfdesktop_file_icon_update_info(file_icon, NULL);
+}
+
 static inline void
 xfdesktop_volume_icon_invalidate_pixbuf(XfdesktopVolumeIcon *icon)
 {
@@ -222,12 +251,51 @@ xfdesktop_volume_icon_peek_label(XfdesktopIcon *icon)
     return thunar_vfs_volume_get_name(XFDESKTOP_VOLUME_ICON(icon)->priv->volume);
 }
 
-static gboolean
-xfdesktop_volume_icon_is_drop_dest(XfdesktopIcon *icon)
+static GdkDragAction
+xfdesktop_volume_icon_get_allowed_drag_actions(XfdesktopIcon *icon)
 {
     XfdesktopVolumeIcon *volume_icon = XFDESKTOP_VOLUME_ICON(icon);
-    /* FIXME: return TRUE, and attempt to mount later if it isn't? */
-    return thunar_vfs_volume_is_mounted(volume_icon->priv->volume);
+    
+    /* volume icons more or less represent the volume's mount point, usually
+     * (hopefully) a local path.  so when it's mounted, we certainly can't move
+     * the mount point, but copying and linking should be OK.  when not mounted,
+     * we should just disallow everything, since, even if its ThunarVfsInfo
+     * is valid, we can't guarantee it won't change after mounting. */
+    
+    /* FIXME: should i allow all actions if not mounted as well, and try to
+     * mount and resolve on drop? */
+    
+    if(thunar_vfs_volume_is_mounted(volume_icon->priv->volume)) {
+        const ThunarVfsInfo *info = xfdesktop_file_icon_peek_info(XFDESKTOP_FILE_ICON(icon));
+        if(info) {
+            if(info->flags & THUNAR_VFS_FILE_FLAGS_READABLE)
+                return GDK_ACTION_COPY | GDK_ACTION_LINK;
+            else
+                return GDK_ACTION_LINK;
+        }
+    }
+    
+    return 0;
+}
+
+static GdkDragAction
+xfdesktop_volume_icon_get_allowed_drop_actions(XfdesktopIcon *icon)
+{
+    XfdesktopVolumeIcon *volume_icon = XFDESKTOP_VOLUME_ICON(icon);
+    
+    /* if not mounted, it doesn't really make sense to allow any operations
+     * here.  if mounted, we should allow everything if it's writable. */
+    
+    /* FIXME: should i allow all actions if not mounted as well, and try to
+     * mount and resolve on drop? */
+    
+    if(thunar_vfs_volume_is_mounted(volume_icon->priv->volume)) {
+        const ThunarVfsInfo *info = xfdesktop_file_icon_peek_info(XFDESKTOP_FILE_ICON(icon));
+        if(info && info->flags & THUNAR_VFS_FILE_FLAGS_WRITABLE)
+            return GDK_ACTION_MOVE | GDK_ACTION_COPY | GDK_ACTION_LINK;
+    }
+    
+    return 0;
 }
 
 static void
@@ -242,7 +310,10 @@ xfdesktop_volume_icon_drag_job_error(ThunarVfsJob *job,
                                                                         "--xfdesktop-file-icon-action"));
     const ThunarVfsInfo *src_info = xfdesktop_file_icon_peek_info(src_file_icon);
     
-    g_return_if_fail(volume_icon && src_file_icon);
+    g_return_if_fail(volume_icon);
+    
+    if(!src_file_icon)
+        return;
     
     xfdesktop_file_utils_handle_fileop_error(NULL, src_info,
                                              volume_icon->priv->info,
@@ -255,7 +326,10 @@ xfdesktop_volume_icon_interactive_job_ask(ThunarVfsJob *job,
                                           ThunarVfsInteractiveJobResponse choices,
                                           gpointer user_data)
 {
-    return xfdesktop_file_utils_interactive_job_ask(NULL, message, choices);
+    GtkWidget *icon_view = xfdesktop_icon_peek_icon_view(XFDESKTOP_ICON(user_data));
+    GtkWidget *toplevel = gtk_widget_get_toplevel(icon_view);
+    return xfdesktop_file_utils_interactive_job_ask(GTK_WINDOW(toplevel),
+                                                    message, choices);
 }
 
 static void
@@ -278,7 +352,7 @@ xfdesktop_volume_icon_drag_job_finished(ThunarVfsJob *job,
     g_object_unref(G_OBJECT(volume_icon));
 }
 
-static XfdesktopIconDragResult
+static gboolean
 xfdesktop_volume_icon_do_drop_dest(XfdesktopIcon *icon,
                                  XfdesktopIcon *src_icon,
                                  GdkDragAction action)
@@ -292,21 +366,25 @@ xfdesktop_volume_icon_do_drop_dest(XfdesktopIcon *icon,
     
     DBG("entering");
     
-    g_return_val_if_fail(volume_icon && src_file_icon,
-                         XFDESKTOP_ICON_DRAG_FAILED);
-    g_return_val_if_fail(xfdesktop_volume_icon_is_drop_dest(icon),
-                         XFDESKTOP_ICON_DRAG_FAILED);
+    g_return_val_if_fail(volume_icon && src_file_icon, FALSE);
+    g_return_val_if_fail(xfdesktop_volume_icon_get_allowed_drop_actions(icon),
+                         FALSE);
     
     src_info = xfdesktop_file_icon_peek_info(src_file_icon);
     if(!src_info)
-        return XFDESKTOP_ICON_DRAG_FAILED;
+        return FALSE;
+    
+    if(thunar_vfs_path_is_root(src_info->path))
+        return FALSE;
     
     name = thunar_vfs_path_get_name(src_info->path);
-    g_return_val_if_fail(name, XFDESKTOP_ICON_DRAG_FAILED);
+    if(!name)
+        return FALSE;
         
     dest_path = thunar_vfs_path_relative(volume_icon->priv->info->path,
                                          name);
-    g_return_val_if_fail(dest_path, XFDESKTOP_ICON_DRAG_FAILED);
+    if(!dest_path)
+        return FALSE;
     
     switch(action) {
         case GDK_ACTION_MOVE:
@@ -364,11 +442,11 @@ xfdesktop_volume_icon_do_drop_dest(XfdesktopIcon *icon,
         
         g_object_ref(G_OBJECT(job));
         
-        return XFDESKTOP_ICON_DRAG_SUCCEEDED_NO_ACTION;
+        return TRUE;
     } else
-        return XFDESKTOP_ICON_DRAG_FAILED;
+        return FALSE;
     
-    return XFDESKTOP_ICON_DRAG_FAILED;
+    return FALSE;
 }
 
 static G_CONST_RETURN gchar *
@@ -382,20 +460,33 @@ xfdesktop_volume_icon_peek_tooltip(XfdesktopIcon *icon)
     /* FIXME: something different? */
     
     if(!volume_icon->priv->tooltip) {
-        gchar mod[64], *kind, sizebuf[64], *size;
-        struct tm *tm = localtime(&volume_icon->priv->info->mtime);
-
-        strftime(mod, 64, "%Y-%m-%d %H:%M:%S", tm);
-        kind = xfdesktop_file_utils_get_file_kind(volume_icon->priv->info, NULL);
-        thunar_vfs_humanize_size(volume_icon->priv->info->size, sizebuf, 64);
-        size = g_strdup_printf(_("%s (%" G_GINT64_FORMAT " Bytes)"), sizebuf,
-                              (gint64)volume_icon->priv->info->size);
+        gchar freebuf[128], totbuf[128], *space;
+        ThunarVfsPath *path;
+        gchar mntpnt[THUNAR_VFS_PATH_MAXSTRLEN] = { 0, };
+        ThunarVfsFileSize size;
+        struct statvfs stfs;
         
-        volume_icon->priv->tooltip = g_strdup_printf(_("Kind: %s\nModified:%s\nSize: %s"),
-                                                   kind, mod, size);
+        path = thunar_vfs_volume_get_mount_point(volume_icon->priv->volume);
+        if(path
+           && thunar_vfs_path_to_string(path, mntpnt, sizeof(mntpnt), NULL) > 0
+           && thunar_vfs_info_get_free_space(volume_icon->priv->info, &size)
+           && !statvfs(mntpnt, &stfs))
+        {
+            thunar_vfs_humanize_size(size, freebuf, sizeof(freebuf));
+            thunar_vfs_humanize_size((ThunarVfsFileSize)(stfs.f_blocks * stfs.f_bsize),
+                                     totbuf, sizeof(totbuf));
+            space = g_strdup_printf(_("%s (%s total)"), freebuf, totbuf);
+        } else
+            space = g_strdup(_("(unknown)"));
         
-        g_free(kind);
-        g_free(size);
+        volume_icon->priv->tooltip = g_strdup_printf(_("Kind: Removable Volume\n"
+                                                       "Mount Point: %s\n"
+                                                       "Free Space: %s"),
+                                                     *mntpnt ? mntpnt
+                                                             : _("(unknown)"),
+                                                     space);
+        
+        g_free(space);
     }
     
     return volume_icon->priv->tooltip;
@@ -406,33 +497,17 @@ xfdesktop_volume_icon_menu_toggle_mount(GtkWidget *widget,
                                         gpointer user_data)
 {
     XfdesktopVolumeIcon *icon = XFDESKTOP_VOLUME_ICON(user_data);
-    GtkWidget *toplevel = NULL;
+    GtkWidget *icon_view = xfdesktop_icon_peek_icon_view(XFDESKTOP_ICON(icon));
+    GtkWidget *toplevel = gtk_widget_get_toplevel(icon_view);
     GError *error = NULL;
     gboolean is_mount;
     
-    /* FIXME */
-    /* toplevel = gtk_widget_get_toplevel(GTK_WIDGET(fmanager->priv->icon_view)); */
-    
     is_mount = !thunar_vfs_volume_is_mounted(icon->priv->volume);
     
     if(!is_mount)
         thunar_vfs_volume_unmount(icon->priv->volume, toplevel, &error);
-    else {
-        if(thunar_vfs_volume_mount(icon->priv->volume, toplevel, &error)) {
-            const ThunarVfsInfo *info = icon->priv->info;
-            ThunarVfsPath *new_path = thunar_vfs_volume_get_mount_point(icon->priv->volume);
-            
-            if(!info || !thunar_vfs_path_equal(info->path, new_path)) {
-                ThunarVfsInfo *new_info = thunar_vfs_info_new_for_path(new_path,
-                                                                       NULL);
-                if(new_info) {
-                    xfdesktop_file_icon_update_info(XFDESKTOP_FILE_ICON(icon),
-                                                    new_info);
-                    thunar_vfs_info_unref(new_info);
-                }
-            }
-        }
-    }
+    else
+        thunar_vfs_volume_mount(icon->priv->volume, toplevel, &error);
     
     if(error) {
         gchar *primary = g_markup_printf_escaped(is_mount ? _("Unable to mount \"%s\":")
@@ -452,16 +527,14 @@ xfdesktop_volume_icon_menu_eject(GtkWidget *widget,
                                  gpointer user_data)
 {
     XfdesktopVolumeIcon *icon = XFDESKTOP_VOLUME_ICON(user_data);
-    GtkWidget *toplevel = NULL;
     GError *error = NULL;
-    
-    /* FIXME */
-    /* toplevel = gtk_widget_get_toplevel(GTK_WIDGET(fmanager->priv->icon_view)); */
+    GtkWidget *icon_view = xfdesktop_icon_peek_icon_view(XFDESKTOP_ICON(icon));
+    GtkWidget *toplevel = gtk_widget_get_toplevel(icon_view);
     
     if(!thunar_vfs_volume_eject(icon->priv->volume, toplevel, &error)) {
         gchar *primary = g_markup_printf_escaped(_("Unable to eject \"%s\":"),
                                                  thunar_vfs_volume_get_name(icon->priv->volume));
-        xfce_message_dialog(toplevel ? GTK_WINDOW(toplevel) : NULL,
+        xfce_message_dialog(GTK_WINDOW(toplevel),
                             _("Eject Failed"), GTK_STOCK_DIALOG_ERROR,
                             primary, error->message,
                             GTK_STOCK_CLOSE, GTK_RESPONSE_ACCEPT, NULL);
@@ -478,84 +551,14 @@ xfdesktop_volume_icon_menu_properties(GtkWidget *widget,
     xfdesktop_file_properties_dialog_show(NULL, icon, NULL);
 }
 
-static void
-xfdesktop_volume_icon_menu_open(GtkWidget *widget,
-                                gpointer user_data)
-{
-    XfdesktopVolumeIcon *icon = XFDESKTOP_VOLUME_ICON(user_data);
-    const ThunarVfsInfo *info;
-    ThunarVfsVolume *volume = (ThunarVfsVolume *)xfdesktop_volume_icon_peek_volume(icon);
-    GError *error = NULL;
-    ThunarVfsPath *new_path = NULL;
-    
-    info = xfdesktop_file_icon_peek_info(XFDESKTOP_FILE_ICON(icon));
-    
-    if(!thunar_vfs_volume_is_mounted(volume)) {
-        /* FIXME: need toplevel parent here */
-        if(!thunar_vfs_volume_mount(volume, NULL, &error)) {
-            gchar *primary = g_markup_printf_escaped(_("Unable to mount \"%s\":"),
-                                                     thunar_vfs_volume_get_name(volume));
-            GtkWidget *dlg = xfce_message_dialog_new(NULL, _("Mount Failed"),
-                                                     GTK_STOCK_DIALOG_ERROR,
-                                                     primary,
-                                                     error ? error->message
-                                                           : _("Unknown error."),
-                                                     GTK_STOCK_CLOSE,
-                                                     GTK_RESPONSE_ACCEPT, NULL);
-            gtk_window_set_screen(GTK_WINDOW(dlg), icon->priv->gscreen);
-            gtk_dialog_run(GTK_DIALOG(dlg));
-            gtk_widget_destroy(dlg);
-            g_free(primary);
-            g_error_free(error);
-        }
-    }
-    
-    new_path = thunar_vfs_volume_get_mount_point(volume);
-        
-    if(new_path && (!info
-                    || !thunar_vfs_path_equal(info->path, new_path)))
-    {
-        ThunarVfsInfo *new_info = thunar_vfs_info_new_for_path(new_path,
-                                                               NULL);
-        if(new_info) {
-            xfdesktop_file_icon_update_info(XFDESKTOP_FILE_ICON(icon),
-                                            new_info);
-            thunar_vfs_info_unref(new_info);
-        }
-    }
-    
-    info = xfdesktop_file_icon_peek_info(XFDESKTOP_FILE_ICON(icon));
-    if(!info || !new_path) {
-        /* if |new_path| is NULL, but |info| isn't, it's possible we have
-         * a stale |info|, and we shouldn't continue */
-        return;
-    }
-    
-    
-    if(!xfdesktop_file_utils_launch_external(info, icon->priv->gscreen)) {
-        gchar *primary = g_markup_printf_escaped(_("Unable to launch \"%s\":"),
-                                                 info->display_name);
-        GtkWidget *dlg = xfce_message_dialog_new(NULL, _("Launch Error"),
-                                                 GTK_STOCK_DIALOG_ERROR,
-                                                 primary,
-                                                 _("The associated application could not be found or executed."),
-                                                 GTK_STOCK_CLOSE,
-                                                 GTK_RESPONSE_ACCEPT, NULL);
-        gtk_window_set_screen(GTK_WINDOW(dlg), icon->priv->gscreen);
-        gtk_dialog_run(GTK_DIALOG(dlg));
-        gtk_widget_destroy(dlg);
-        g_free(primary);
-    }
-}
-
-static GtkWidget *
-xfdesktop_volume_icon_get_popup_menu(XfdesktopIcon *icon)
+static gboolean
+xfdesktop_volume_icon_populate_context_menu(XfdesktopIcon *icon,
+                                            GtkWidget *menu)
 {
     XfdesktopVolumeIcon *volume_icon = XFDESKTOP_VOLUME_ICON(icon);
     ThunarVfsVolume *volume = volume_icon->priv->volume;
-    GtkWidget *menu, *mi, *img;
-    
-    menu = gtk_menu_new();
+    GtkWidget *mi, *img;
+    gboolean mounted, ejectable;
     
     img = gtk_image_new_from_stock(GTK_STOCK_OPEN, GTK_ICON_SIZE_MENU);
     gtk_widget_show(img);
@@ -563,50 +566,49 @@ xfdesktop_volume_icon_get_popup_menu(XfdesktopIcon *icon)
     gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(mi), img);
     gtk_widget_show(mi);
     gtk_menu_shell_append(GTK_MENU_SHELL(menu), mi);
-    g_signal_connect(G_OBJECT(mi), "activate",
-                     G_CALLBACK(xfdesktop_volume_icon_menu_open), icon);
+    g_signal_connect_swapped(G_OBJECT(mi), "activate",
+                             G_CALLBACK(xfdesktop_icon_activated), icon);
     
     mi = gtk_separator_menu_item_new();
     gtk_widget_show(mi);
     gtk_menu_shell_append(GTK_MENU_SHELL(menu), mi);
     
-    mi = gtk_image_menu_item_new_with_mnemonic(_("_Mount Volume"));
-    gtk_widget_show(mi);
-    gtk_menu_shell_append(GTK_MENU_SHELL(menu), mi);
-    if(thunar_vfs_volume_is_mounted(volume))
-        gtk_widget_set_sensitive(mi, FALSE);
-    else {
+    mounted = thunar_vfs_volume_is_mounted(volume);
+    ejectable = thunar_vfs_volume_is_ejectable(volume);
+
+    if(!mounted || (mounted && !ejectable)) {
+        if(mounted) {
+            img = gtk_image_new_from_stock(GTK_STOCK_DISCONNECT, GTK_ICON_SIZE_MENU);
+            mi = gtk_image_menu_item_new_with_mnemonic(_("_Unmount Volume"));
+        } else {
+            img = gtk_image_new_from_stock(GTK_STOCK_CONNECT, GTK_ICON_SIZE_MENU);
+            mi = gtk_image_menu_item_new_with_mnemonic(_("_Mount Volume"));
+        }
+        gtk_widget_show(img);
+        gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(mi), img);
+        gtk_widget_show(mi);
+        gtk_menu_shell_append(GTK_MENU_SHELL(menu), mi);
         g_signal_connect(G_OBJECT(mi), "activate",
                          G_CALLBACK(xfdesktop_volume_icon_menu_toggle_mount),
                          icon);
     }
-    
-    mi = gtk_image_menu_item_new_with_mnemonic(_("_Unmount Volume"));
-    gtk_widget_show(mi);
-    gtk_menu_shell_append(GTK_MENU_SHELL(menu), mi);
-    if(!thunar_vfs_volume_is_mounted(volume))
-        gtk_widget_set_sensitive(mi, FALSE);
-    else {
-        g_signal_connect(G_OBJECT(mi), "activate",
-                         G_CALLBACK(xfdesktop_volume_icon_menu_toggle_mount),
-                         icon);
-    }
-    
-    if(thunar_vfs_volume_is_disc(volume)
-       && thunar_vfs_volume_is_ejectable(volume))
-    {
+
+    if(ejectable) {
+        img = gtk_image_new_from_icon_name("media-eject", GTK_ICON_SIZE_MENU);
+        gtk_widget_show(img);
         mi = gtk_image_menu_item_new_with_mnemonic(_("E_ject Volume"));
+        gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(mi), img);
         gtk_widget_show(mi);
         gtk_menu_shell_append(GTK_MENU_SHELL(menu), mi);
         g_signal_connect(G_OBJECT(mi), "activate",
                          G_CALLBACK(xfdesktop_volume_icon_menu_eject),
                          icon);
     }
-    
+
     mi = gtk_separator_menu_item_new();
     gtk_widget_show(mi);
     gtk_menu_shell_append(GTK_MENU_SHELL(menu), mi);
-    
+
     img = gtk_image_new_from_stock(GTK_STOCK_PROPERTIES, GTK_ICON_SIZE_MENU);
     gtk_widget_show(img);
     mi = gtk_image_menu_item_new_with_mnemonic(_("_Properties..."));
@@ -621,7 +623,7 @@ xfdesktop_volume_icon_get_popup_menu(XfdesktopIcon *icon)
                          icon);
     }
     
-    return menu;
+    return TRUE;
 }
 
 
@@ -661,6 +663,26 @@ xfdesktop_volume_icon_update_info(XfdesktopFileIcon *icon,
     xfdesktop_icon_pixbuf_changed(XFDESKTOP_ICON(icon));
 }
 
+static gboolean
+xfdesktop_volume_icon_activated(XfdesktopIcon *icon_p)
+{
+    XfdesktopVolumeIcon *icon = XFDESKTOP_VOLUME_ICON(icon_p);
+    ThunarVfsVolume *volume = (ThunarVfsVolume *)xfdesktop_volume_icon_peek_volume(icon);
+    
+    TRACE("entering");
+    
+    if(!thunar_vfs_volume_is_mounted(volume)) {
+        xfdesktop_volume_icon_menu_toggle_mount(NULL, icon);
+        if(!thunar_vfs_volume_is_mounted(volume))
+            return TRUE;  /* mount failed; halt signal emission */
+    }
+    
+    /* chain up */
+    return XFDESKTOP_ICON_CLASS(xfdesktop_volume_icon_parent_class)->activated(icon_p);
+}
+
+
+
 XfdesktopVolumeIcon *
 xfdesktop_volume_icon_new(ThunarVfsVolume *volume,
                           GdkScreen *screen)
@@ -680,6 +702,10 @@ xfdesktop_volume_icon_new(ThunarVfsVolume *volume,
         /* |path| is owned by |volume|, do not free */
     }
     
+    g_signal_connect(G_OBJECT(volume), "changed",
+                     G_CALLBACK(xfdesktop_volume_icon_volume_changed_cb),
+                     volume_icon);
+    
     return volume_icon;
 }
 
